package fr.unistra.sil.erp.back.repository.category;

import fr.unistra.sil.erp.back.model.Category;
import fr.unistra.sil.erp.back.repository.DatabaseConnectionException;
import fr.unistra.sil.erp.back.repository.DatabaseResourceNotFoundException;
import fr.unistra.sil.erp.back.repository.DatabaseUpdateException;
import fr.unistra.sil.erp.back.repository.ICategoriesRepository;
import fr.unistra.sil.erp.back.repository.SqliteRepository;
import org.springframework.stereotype.Repository;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * SQLite Category SQL queries.
 * @author BEAUVAIS ANTOINE
 */
@Repository
public class SqliteCategoryRepository extends SqliteRepository implements ICategoriesRepository {

    /**
     * Query used to retrieve all categories stored in the database.
     */
    private static final String SQL_GETCATEGORIES =
            "SELECT id, name FROM categories";
    
    /**
     * Query used to retrieve one category stored in the database.
     */
    private static final String SQL_GETCATEGORY =
            "SELECT id, name FROM categories WHERE id = ?";
    
    private static final String SQL_GETLASTCATEGORY =
            "SELECT MAX(id), name FROM categories";
    
    /**
     * Query used to create a new category.
     */
    private static final String SQL_CREATECATEGORY =
            "INSERT INTO categories (name) VALUES (?)";

    public SqliteCategoryRepository(Connection conn) {
        super(conn);
    }

    @Override
    public List<Category> getCategories() {
        ResultSet rs = this.query(SQL_GETCATEGORIES);
        if (rs == null)
            return null;

        List<Category> res = new ArrayList<>();
        try {
            while (rs.next()) {
                Category c = new Category(rs.getInt("id"), rs.getString("name"));
                res.add(c);
            }
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(
                    Level.SEVERE, "Failed to fetch results.", ex);
            return null;
        }

        return res;
    }
    
    public Category getCategory(int id) throws DatabaseResourceNotFoundException
    {
        PreparedStatement ps;
        try {
            ps = this.conn.prepareStatement(SQL_GETCATEGORY);
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(
                    Level.SEVERE, "Failed to connect to database.", ex);
            return null;
        }

        try {
            ps.setInt(1, id);
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(
                    Level.SEVERE, "Failed to set query parameter.", ex);
            return null;
        }

        ResultSet rs;
        try {
            rs = ps.executeQuery();
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(
                    Level.SEVERE, "Failed to fetch stock for item " +
                            id, ex);
            return null;
        }

        Category c;
        try {
            if (rs.next())
                c = new Category(rs.getInt("id"), rs.getString("name"));
            else
                throw new DatabaseResourceNotFoundException("Stock not found.");
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(
                    Level.SEVERE, "Failed to parse results.", ex);
            return null;
        }

        return c;
    }
    
    /**
     * Retrieves the latest category.
     * @return 
     */
    @Override
    public Category getLatestCategory()
    {
        ResultSet rs = this.query(SQL_GETCATEGORIES);
        if (rs == null)
            return null;

        Category c = null;
        try {
            while (rs.next()) {
                c = new Category(rs.getInt("id"), rs.getString("name"));
            }
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(
                    Level.SEVERE, "Failed to fetch results.", ex);
            return null;
        }

        return c;
    }
    
    @Override
    public Category createCategory(Category c)
            throws DatabaseConnectionException, DatabaseUpdateException
    {
        PreparedStatement ps;
        try {
            ps = this.conn.prepareStatement(SQL_CREATECATEGORY);
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(
                    Level.SEVERE, "Failed to connect to database.", ex);
            throw new DatabaseConnectionException("Failed to connect to DB.");
        }

        try {
            ps.setString(1, c.getName());
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName()).log(
                    Level.SEVERE, "Failed to set query parameter.", ex);
            throw new DatabaseUpdateException("Failed to set query params.");
        }

        try {
            ps.execute();
        } catch (SQLException ex) {
            Logger.getLogger(this.getClass().getName())
                    .log(Level.SEVERE, "Failed to execute query.", ex);
            throw new DatabaseUpdateException("Failed to execute query.");
        }
        
        return c;
    }
}
